#!/usr/bin/env python3
# python_intro.py
# Glenn G. Chappell
# 1 Mar 2013
#
# For CS 331 Spring 2013
# Code from 3/1: Introductory Python

import itertools       # for itertools.islice
import urllib.request  # for urllib.request.urlopen


# ***** Some Programming *****


# Basic function

# fibo - has docstring with doctests
def fibo(n):
    """Return nth Fibonaci number.

    >>> fibo(0)
    0
    >>> fibo(1)
    1
    >>> fibo(6)
    8

    """
    prev = 1
    curr = 0
    for i in range(n):
        next = prev + curr
        prev = curr
        curr = next
    return curr

# Try:
#   >>> import python_intro
#   >>> python_intro.fibo(8)
#   >>> import doctest
#   >>> doctest.testmod(python_intro)
#   >>> doctest.testmod(python_intro,verbose=True)

# Or, for you *ix command-line users ...
# Try:
#   $ python3 -m doctest python_intro.py
#   $ python3 -m doctest -v python_intro.py


def print_fibos(k):
    """Print first k Fibonacci numbers, 0,1,1,..., one on each line."""

    for n in range(k):
        print(fibo(n))

# Try:
#   >>> import python_intro
#   >>> python_intro.print_fibos(10)


# Generator

def fibo_gen():
    """Generator. Yield Fibonacci numbers, 0,1,1,2,..."""

    prev = 1
    curr = 0
    while (True):
        yield curr
        new = prev + curr
        prev = curr
        curr = new


def print_fibos_from_gen(k):
    """Print first k Fibonacci numbers, 0,1,1,..., one on each line."""

    for f in itertools.islice(fibo_gen(), k):
        print(f)


# Try:
#   >>> import python_intro
#   >>> python_intro.print_fibos_from_gen(10)


# Main program

if __name__ == "__main__":
    k = 15
    print("First", k, "Fibonacci numbers:")
    print_fibos_from_gen(k)
    print()
    print("First", k, "Fibonacci numbers, as a list:")
    print(list(itertools.islice(fibo_gen(), k)))


# High-level coding

def get_webpage(url):
    """Get webpage from given URL, return text as string."""

    with urllib.request.urlopen(url) as f:
        contents = f.read().decode()
    return contents

# Try:
#   >>> import python_intro
#   >>> w = python_intro.get_webpage("http://www.google.com/")
#   >>> print(w[0:30])

